/* TI C6X ELF attributes.
   Copyright (C) 2010-2021 Free Software Foundation, Inc.

   This file is part of BFD, the Binary File Descriptor library.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
   MA 02110-1301, USA.  */

/* Define the TAG macro before including this file; it takes a tag
   name and value.  */

TAG(Tag_ISA, 4)
TAG(Tag_ABI_wchar_t, 6)
TAG(Tag_ABI_stack_align_needed, 8)
TAG(Tag_ABI_stack_align_preserved, 10)
TAG(Tag_ABI_DSBT, 12)
TAG(Tag_ABI_PID, 14)
TAG(Tag_ABI_PIC, 16)
TAG(Tag_ABI_array_object_alignment, 18)
TAG(Tag_ABI_array_object_align_expected, 20)
TAG(Tag_ABI_compatibility, 32)
TAG(Tag_ABI_conformance, 67)
